#include <iostream>
#include <vector>
#include <cassert>

#include <cairo/cairo.h>

using namespace std;

int main(int argc, char** argv) {

	unsigned width = 100;
	unsigned height = 100;
	
	std::vector<std::uint32_t> buf(width * height * sizeof(std::uint32_t));
	
	cairo_surface_t* surface = cairo_image_surface_create_for_data(
			reinterpret_cast<unsigned char*>(&*buf.begin()),
			CAIRO_FORMAT_ARGB32,
			width,
			height,
			width * sizeof(std::uint32_t)
		);
	assert(surface);
	
	cairo_t* cr = cairo_create(surface);
	assert(cr);
	assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
	
	
	{
		cairo_matrix_t m;
		
		m.x0 = 1.81202;
		m.xx = 0.739599;
		m.xy = 0;
		m.y0 = 0.258137;
		m.yx = 0;
		m.yy = 0.737535;
		cairo_set_matrix(cr, &m);
		assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
	}
	
	cairo_rectangle(cr, -100000, -100000, 200000, 200000);
	assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
	
	{
		cairo_matrix_t m;

		m.x0 = -3.89731e+07;
		m.xx = 0.739599;
		m.xy = 1.32914e-05;
		m.y0 = -5.61557e+07;
		m.yx = 0.737535;
		m.yy = 1.91526e-05;

		cairo_set_matrix(cr, &m);
		assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
	}
	
	cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
	assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
	
	auto pat = cairo_pattern_create_radial(
			1,
			1,
			0,
			1,
			1,
			-6481920.0
		);
	assert(pat);
	assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
	
	cairo_pattern_add_color_stop_rgba(pat, 0, 1, 1, 1, 1);
	assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
	cairo_pattern_add_color_stop_rgba(pat, 0.5, 1, 0.1, 1, 0.5);
	assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
	
	cairo_pattern_set_extend(pat, CAIRO_EXTEND_PAD);
	assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
	
	cairo_set_source(cr, pat);
	assert(cairo_status(cr) == CAIRO_STATUS_SUCCESS);
	
	cairo_fill_preserve(cr);
	assert(cairo_status(cr) != CAIRO_STATUS_NO_MEMORY);
	
	return 0;
}

